@@ -1,19 +1,16 @@
 # coding: utf-8
-from __future__ import absolute_import, unicode_literals, print_function
+from __future__ import unicode_literals, print_function
+
+import gc
 
 from FL import fl
-import gc
-import os
-
-resources_path = os.path.join(os.path.dirname(__file__), 'resources')
-if resources_path not in os.environ['PATH']:
-  os.environ['PATH'] += '%s;' % resources_path
-
 from . import core
 
 show_default_optimize_code_points = core.show_default_optimize_code_points
 
-__version__ = '0.8.1'
+__author__ = 'Jameson R Spires'
+__version__ = '0.8.5'
+__license__ = 'MIT'
 __doc__ = """
 VFB2UFO3
 DESCRIPTION
@@ -24,15 +21,17 @@ VFB2UFO3 is primarily intended to create scaled UFO instances from a > 1000 UPM
 multiple master FontLab `.vfb` font with PostScript outlines for use with the
 AFDKO tools for creating binary fonts while still working with Windows FontLab
 5.2. The most significant non-trivial change that will occur in export is the
-renaming of kerning glyph groups (FontLab classes). Providing a `.flc`
-(FontLab-class) file can speed up conversion significantly when font groups are
-not identifiable as first/second from their name. All glyph hints/links are
-ignored by default; a batch command can be created for use with `psautohint`,
-which supports hinting outlines with decimal coordinates.
+renaming of kerning glyph groups (FontLab classes). Providing an `.flc`
+(FontLab-class) file can speed up conversion significantly when font groups
+are not identifiable as first/second from their name. Glyph hints/links can be
+exported using 1 of 3 formats; the UFO3 specification, Adobe AutoHint v1, or
+Adobe AutoHint v2. The hinting information is the same, however makeOTF does
+not support the UFO3 specification.
 
 VFB2UFO3 also quickly creates UFO masters from multiple master `.vfb` fonts.
-This is the default export option when the `instance_values` option is not
-supplied.
+This is the default behavior when the `instance_values` option is not
+supplied, or building a `.designspace` file for use in building a variable
+font.
 
 INSTALLATION
 PyPi
@@ -43,10 +42,8 @@ Download the lastest release FontLab installer (`.flw`) file and drag it into
 the FontLab main window then restart FontLab or reset macro system.
 
 ZIP Archive
-Download and extract the latest release `.zip` file and move the extracted
-files to your FontLab Macros folder. The default directory structure is shown
-below.
-
+Download and extract the latest release ZIP file and move the extracted files
+to your FontLab Macros folder. The default directory structure is shown below.
 
 ```
 [user folder]
@@ -57,26 +54,17 @@ below.
               └─ System
                  └─ Modules
                     └─ vfb2ufo3
-                       ├─ resources
-                       │  ├─ libgcc_s_dw2-1.dll
-                       │  ├─ libgomp-1.dll
-                       │  ├─ libstdc++-6.dll
-                       │  ├─ libwinpthread-1.dll
-                       │  └─ zlib1.dll
                        ├─ __init__.py
                        ├─ core.pyd
-                       ├─ designspace.pyd
-                       ├─ fdk.pyd
                        ├─ fea.pyd
-                       ├─ fontinfo.pyd
                        ├─ glif.pyd
-                       ├─ groups.pyd
-                       ├─ kern.pyd
-                       ├─ mark.pyd
-                       ├─ plist.pyd
-                       ├─ tools.pyd
+                       ├─ libgcc_s_dw2-1.dll
+                       ├─ libgomp-1.dll
+                       ├─ libstdc++-6.dll
+                       ├─ libwinpthread-1.dll
                        ├─ user.py
-                       └─ vfb.pyd
+                       ├─ vfb.pyd
+                       └─ zlib1.dll
 ```
 
 REQUIREMENTS
@@ -84,30 +72,31 @@ This package has no Python dependencies outside of the standard library. It is
 written in C++ and Cython. The submodules are compiled into `.pyd` extension
 modules. To recompile the submodules, the PyPi `cython` package and a compiler
 for Cython to utilize during extension module compilation will be required.
+The modules are currently compiled using a MSYS2 version of GCC.
 
 Extension modules compiled from C++ require several DLLs from the GCC which are
-included in the release `.zip` archive, FontLab installer, and PyPi package.
+included in the release ZIP archive, FontLab installer, and PyPi package.
 
 OPTIONAL
-  cython
-  pip install cython
-  https://github.com/cython/cython
+cython
+pip install cython
+https://github.com/cython/cython
 
-  AFDKO
-  pip install afdko
-  https://github.com/adobe-type-tools/afdko
+AFDKO
+pip install afdko
+https://github.com/adobe-type-tools/afdko
 
-  psautohint
-  pip install psautohint
-  https://github.com/adobe-type-tools/psautohint
+psautohint
+pip install psautohint
+https://github.com/adobe-type-tools/psautohint
 
-  MinGW 32-bit/i686 GCC >= 9.3.0
-  http://winlibs.com
-  https://www.msys2.org
-  http://mingw.org
+MinGW 32-bit/i686 GCC >= 9.3.0
+http://winlibs.com
+https://www.msys2.org
+http://mingw-w64.org
 
-  {fmt}
-  https://github.com/fmtlib/fmt
+{fmt}
+https://github.com/fmtlib/fmt
 
 FUNCTIONALITY
 UFO output is produced without changes to the source font. If a specific
@@ -126,16 +115,15 @@ absolute will be ignored. The default output path is the user's Desktop.
 A dictionary of attributes may be suppled via the `instance_attributes` option.
 These attributes should consist of keys from the UFO specification and they
 must match the data type in the specification. Not all fontinfo attributes are
-configurable; please see `CONFIGURABLE_ATTRS` in the `fontinfo.pxi` source file
-for a list of attributes which will be checked and updated to the UFO(s) during
-creation.
-
+configurable; please see `CONFIGURABLE_ATTRS` in the `fontinfo.pxi` source
+file for a list of attributes which will be checked and updated to the UFO(s)
+during creation.
 
 ```
 -------------------------------------------------------------------------------
 EXAMPLE UFO GENERATION SCRIPT FOR SINGLE-AXIS MULTIPLE MASTER .VFB FONT
 -------------------------------------------------------------------------------
-#FLM: write ufo
+# FLM: write ufo
 # coding: utf-8
 from __future__ import absolute_import, unicode_literals
 
@@ -203,21 +191,22 @@ the list values are numerical, the values will be converted to single-element
 lists:
 
 2-axis font
-  `instance_values = [[0, 1000], [200, 1000]]`
 
-  `instance_values = [(0, 1000), (200, 1000)]`
+`instance_values = [[0, 1000], [200, 1000]]`
+
+`instance_values = [(0, 1000), (200, 1000)]`
 
 1-axis font
-  `instance_values = [[0], [200]]`
 
-  `instance_values = [(0,), (200,)]`
+`instance_values = [[0], [200]]`
 
-  `instance_values = [0, 200]` becomes `[[0], [200]]`
+`instance_values = [(0,), (200,)]`
 
+`instance_values = [0, 200]` becomes `[[0], [200]]`
 If the optional lists of names and/or attributes are provided in addition to a
 list of values, they must be of the same length as the values list. A
-`ValueError` will be raised if the lengths do not match, since this will likely
-produce undesirable results:
+`ValueError` will be raised if the lengths do not match, since this will
+likely produce undesirable results:
 
 Correct:
 
@@ -237,8 +226,8 @@ GLYPH OPTIONS
 Glyph scaling is independent from the `.vfb` instance itself.
 
 Glyph decomposition and overlap removal is optional and occurs after instances
-are generated. This option is intended for final output when a binary font will
-be created from the UFO.
+are generated. This option is intended for final output when a binary font
+will be created from the UFO.
 
 By default, when decomposing and removing overlaps from glyph outlines for
 export, GLIF files for glyphs containing components will be built using
@@ -253,50 +242,50 @@ to `False`.
 To specify explicitly which glyphs to decompose and/or remove overlaps, the
 `glyphs_decompose_names` and `glyphs_remove_overlap_names` options should be
 supplied. The supplied names must match exactly with the glyph names in the
-target font. The `glyphs_optimize` option should be disabled if using either of
-these options.
+target font. The `glyphs_optimize` option should be disabled if using either
+of these options.
 
 Omit glyphs from instance
-  A list of glyph suffixes and/or glyph names can be supplied that should be
-  omitted from the instance UFO via the `glyphs_omit_suffixes` and
-  `glyphs_omit_names` options, respectively.
 
+A list of glyph suffixes and/or glyph names can be supplied that should be
+omitted from the instance UFO via the `glyphs_omit_suffixes` and
+`glyphs_omit_names` options, respectively.
 Optimize glyph name and code point lists
-  For output intended for makeOTF where construction glyphs are being omitted
-  via `glyphs_omit_list` or `glyphs_omit_suffixes_list`, enabling
-  `glyphs_optimize_makeotf` will decompose and remove overlaps for the omitted
-  glyphs where needed. Overlaps will be removed for all other base components
-  but remain in component-form. This will decrease UFO creation time and allow
-  makeOTF to more efficiently subroutinize the outlines and reduce .otf
-  compilation time.
 
-  If removing overlaps, the default list of code points for glyphs to be
-  constructed in the above manner is composed of glyphs that normally have no
-  overlapping components. The list is located in the `core.pxi` source file
-  with the character representations for these code points shown below.
+For output intended for makeOTF where construction glyphs are being omitted via
+`glyphs_omit_list` or `glyphs_omit_suffixes_list`, enabling
+`glyphs_optimize_makeotf` will decompose and remove overlaps for the omitted
+glyphs where needed. Overlaps will be removed for all other base components
+but remain in component-form. This will decrease UFO creation time and allow
+makeOTF to more efficiently subroutinize the outlines and reduce .otf
+compilation time.
 
-  The code points from the default code point list can be shown by running the
-  `vfb2ufo3.show_default_optimize_code_points()` function. The code points will
-  print to the FontLab output window and can then be copied into a text editor
-  and edited as needed.
+If removing overlaps, the default list of code points for glyphs to be
+constructed in the above manner is composed of glyphs that normally have no
+overlapping components. The list is located in the `core.pxi` source file with
+the character representations for these code points shown below.
 
-  A user-supplied code point list (`glyphs_optimize_code_points`) can be a list
-  of `'0x00ac'`-format strings, integers in hexadecimal-form (`0x00ac`), or
-  numeric integers (`172`); values which cannot be converted to an integer will
-  raise a `ValueError`. Only the first code point in each glyph's list of code
-  points (FontLab glyph unicodes attribute) is checked for code point set
-  membership.
+The code points from the default code point list can be shown by running the
+`vfb2ufo3.show_default_optimize_code_points()` function. The code points will
+print to the FontLab output window and can then be copied into a text editor
+and edited as needed.
 
-  A user-supplied glyph name list (`glyphs_optimize_names`) can be supplied to
-  supplement the code point list for glyphs. Any glyphs containing components
-  that do not overlap should be added to this list.
+A user-supplied code point list (`glyphs_optimize_code_points`) can be a list
+of `'0x00ac'`-format strings, integers in hexadecimal-form (`0x00ac`), or
+numeric integers (`172`); values which cannot be converted to an integer will
+raise a `ValueError`. Only the first code point in each glyph's list of code
+points (FontLab glyph unicodes attribute) is checked for code point set
+membership.
 
-  Small case variants of the code points in the code point list will be also
-  added to code point list assuming they end with `.sc`, `.smcp`, or `.c2sc`
-  suffixes.
+A user-supplied glyph name list (`glyphs_optimize_names`) can be supplied to
+supplement the code point list for glyphs. Any glyphs containing components
+that do not overlap should be added to this list.
+
+Small case variants of the code points in the code point list will be also
+added to code point list assuming they end with `.sc`, `.smcp`, or `.c2sc`
+suffixes.
 
 `OPTIMIZE_CODE_POINTS`
-
 ```
 Latin
 À Á Â Ã Ä Ā Ă Ǣ Ǽ Ȁ Ȃ Ǎ Ȧ
@@ -375,8 +364,8 @@ Greek Mono- and Polytonic
 
 Features options
 Font groups can be added to the `features.fea` file on export by setting
-`features_import_groups` to `True` (if `afdko_parts` is enabled, this will also
-be enabled). The font's features are neither formatted nor checked for
+`features_import_groups` to `True` (if `afdko_parts` is enabled, this will
+also be enabled). The font's features are neither formatted nor checked for
 correctness. Users are responsible for moving referenced feature files from
 `include()` statements to the chosen output directory. Also see KERN FEATURE
 OPTIONS and MARK FEATURE OPTIONS below for `kern` and `mark` feature options.
@@ -384,10 +373,10 @@ OPTIONS and MARK FEATURE OPTIONS below for `kern` and `mark` feature options.
 Hint options
 If enabled, the `glyph_hints` option converts hints on export (glyph links are
 converted to hints). By default, hints will be created following the UFO3
-public.postscript.hints specification. The `glyphs_hints_afdko_v1` and
+public.postscript.hints specification.  The `glyphs_hints_afdko_v1` and
 `glyphs_hints_afdko_v2` options enables hints compatible with MakeOTF.
 
-`glyphs_hints_vertical_only` ignored horizontal hints/links
+`glyphs_hints_vertical_only` ignores horizontal hints/links
 
 Kern feature options
 Kern values will be scaled in parity with the output UFO. This scaling is
@@ -446,10 +435,11 @@ file.
 
 FontLab kern classes without a key glyph in a provided `.flc` file or in the
 master font's kern classes will have the first glyph in the FontLab class
-marked as the key glyph and it will be noted in the output console. If there is
-more than one glyph marked as a key glyph, the first marked glyph is considered
-the key glyph. These key glyph operations may affect the kerning. An imported
-`groups.plist` infers that the glyph in the group name is the key glyph.
+marked as the key glyph and it will be noted in the output console. If there
+is more than one glyph marked as a key glyph, the first marked glyph is
+considered the key glyph. These key glyph operations may affect the kerning.
+An imported `groups.plist` infers that the glyph in the group name is the key
+glyph.
 
 Kern group naming recommendations
 
@@ -493,10 +483,10 @@ AFDKO options
 The `OS/2`, `hhea`, `head`, and `name` tables will be added to the features
 file. The `name` table entry strings will be formatted according to OpenType
 Feature File Specification § 9.e. Any strings unable to be formatted fully for
-each platform's specific encoding restriction (Windows -- `UTF-8`, Macintosh --
-`Mac Roman`) will be formatted to their nearest ASCII equivalent rather than
-omitting any un-encodable characters. The standard library `unicodedata` module
-is used for any entries meeting this criteria.
+each platform's specific encoding restriction (Windows -- `UTF-8`, Macintosh
+-- `Mac Roman`) will be formatted to their nearest ASCII equivalent rather
+than omitting any un-encodable characters. The standard library `unicodedata`
+module is used for any entries meeting this criteria.
 
 GlyphOrderAndAliasDB (GOADB) and FontMenuNameDB files can be generated for use
 with MakeOTF. The GOADB can be provided (`afdko_makeotf_GOADB_path`), derived
@@ -530,8 +520,8 @@ UFOZ options
 UFO instances can be written as a `.ufoz` archive. If you are planning on any
 file transfer operations after creation, transferring a single `.ufoz` file is
 much quicker than the large number of small text files in the generated UFO
-instance(s), especially when transferring through USB. By default, archives are
-written in compressed mode. Compression can be turned off by setting
+instance(s), especially when transferring through USB. By default, archives
+are written in compressed mode. Compression can be turned off by setting
 `ufoz_compress` to `False`.
 
 `.designspace` font options
@@ -539,9 +529,17 @@ A `.designspace` document can be created in place of individual UFO instances.
 A UFO for each master will be generated and the instances will be described in
 the `.designspace` document. A default instance can be described with the
 `designspace_default` option. This value must be a list or tuple with a value
-for each axis in the font. If `glyphs_omit_list` or `glyphs_omit_suffixes_list`
-lists are provided, the glyphs will remain in the source UFOs and a glyph mute
-rule for each glyph to be omitted will be added for each instance.
+for each axis in the font. If `glyphs_omit_list` or
+`glyphs_omit_suffixes_list` lists are provided, the glyphs will remain in the
+source UFOs and a glyph mute rule for each glyph to be omitted will be added
+for each instance.
+
+A list of names can be provided for each layer of the multiple-master source
+font. The name list must be of the same length as the number of layers in the
+source font, and its order is assumed to match the order of the source font
+layers. If a names list is not supplied, the names of the exported masters
+will be constructed using the abbreviated axis names as set in the font
+(accessible via the master font's `axis` attribute).
 
 Benchmarks
 For reference, testing was performed on a Windows 10 machine with an Intel Xeon
@@ -689,6 +687,17 @@ modification of an implementation from Olivier Gay's SHA2 library, which is
 covered under a BSD License.
 
 Version history
+version 0.8.5
+added `scale_factor` and `integer_coordinates` options
+small changes and corrections
+module consolidation
+
+version 0.8.3
+reorganization
+
+version 0.8.2
+small changes and corrections
+
 version 0.8.1
 added `glyphs_optimize_makeotf` option for use with makeOTF
 added `glyphs_decompose_names` and `glyphs_remove_overlap_names` options
@@ -804,8 +813,11 @@ def write_ufo(
   layer=None,
 
   scale_to_upm=1000,
+  scale_factor=1.0,
   scale_auto=True,
 
+  integer_coordinates=False,
+
   instance_values=[],
   instance_names=[],
   instance_attributes=[],
@@ -863,6 +875,8 @@ def write_ufo(
   glyphs_decompose=False,
   glyphs_remove_overlaps=False,
 
+  glyphs_decompose_names=[],
+  glyphs_remove_overlaps_names=[],
   glyphs_omit_names=[],
   glyphs_omit_suffixes=[],
   glyphs_optimize=True,
@@ -890,8 +904,6 @@ def write_ufo(
   if not len(fl):
     raise UserWarning(b'No open fonts')
 
-  fl.output = b''
   options = core.decode_dict(locals())
   core.write_ufo(options)
-  cleanup()
-
+  cleanup()
\ No newline at end of file
